/*  aqotdd - Afghan's Quote of the day server - An RFC865 compliant server.
    Copyright (C) 2000 Afghan Hound Software

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/

/* config.c - Handles finding, reading, and re-reading the configuration file */

#include "main.h"


void  config_find_file();
char* config_getline(char *line,FILE *fptr);
void  config_setdefault();

int   config_havefile;
char *config_filename;
char *config_command_file;


void config_findfile()
{
  int   length = 0;
  FILE *test;

  config_havefile = 1;
  if (config_command_file!=NULL)
    {
      if ((test=fopen(config_command_file,"r"))!=NULL)
	{
	  fclose(test);
	  length=strlen(config_command_file) + 1;
	  config_filename=(char *)xmalloc(length*sizeof(char));
	  strncpy(config_filename, config_command_file,length);
	  free(config_command_file);
	  return;
	}
      else
	{
	  if ((test=fopen(DEF_CONFIGNAME,"r"))!=NULL)
	    {
	      fclose(test);
	      length=strlen(DEF_CONFIGNAME) + 1;
	      config_filename=(char *)xmalloc(length*sizeof(char));
	      strncpy(config_filename, DEF_CONFIGNAME,length);
	    }
	  else
	    {
	      my_message(MSG_ERRFILEC,"Can't find config file, using defaults.","");
	      config_havefile = 0;
	      return;
	    }
	}
    }
   my_message(MSG_CONFIGF,"Using config file",config_filename);
}

void config_parseoption(int c,char **v)
{
  int        length;
  char       next_option;
  extern int opterr; /*Ignore other options silently*/
  
  opterr = 0;
  debug_flag = 0;
  config_command_file=NULL;
  my_message(MSG_INITSTART,"Starting option parsing.","");

  while ((next_option= getopt (c, v, "vhdf:"))!= -1)
    switch(next_option)
      {
      case 'f':
        length=strlen(optarg)+1; /* include \0 */
        config_command_file=(char *)xmalloc(length*sizeof(char));
        strncpy(config_command_file, optarg,(length-1));
        break;
      case 'd':
	debug_flag = 1;
	break;
	case 'v':
        printf("Afghan QOTD Daemon version %s(%s). Copyright (C) 2000 Afghan Hound.\n\n",VERSION,NAME);
	printf("  *** This program comes with ABSOLUTELY NO WARRANTY. *** \n\n");
	printf("You may redistribute copies of Afghan QOTD Daemon under the terms\n");
	printf("of the GNU General Public License. For more information about\n");
	printf("these matters, see the file named COPYING.\n");
	exit(0);
        break;
      case 'h': 
	printf("Afghan QOTD Daemon version %s(%s). Copyright (C) 2000 Afghan Hound.\n",VERSION,NAME);
	printf("Usage:\n");
	printf("     -d          : Turn on debugging mode.\n");
	printf("     -f filename : Specify a configuration file.\n");
	printf("     -h          : print this help.\n");	
	printf("     -v          : print version infomation.\n");
	exit(0);
	break;
      }
  my_message(MSG_INITFINISH,"Finished option parsing.","");
}

void config_setdefault()
{
  use_tcp     =DEF_USE_TCP;use_udp      =DEF_USE_UDP;
  use_off     =DEF_USE_OFF;use_rfc      =DEF_USE_RFC;
  max_con_tcp =DEF_MAX_TCP;max_con_udp  =DEF_MAX_UDP;
  log_msg     =LOG_MESSAGE;abs_min      =ABS_MIN;
  abs_max     =ABS_MAX;    chk_con      =DEF_CHK;
  log_count   =DEF_LOGCOUNT;daemon_flag =DEF_DAEMON;
  mark_time   =DEF_MARKTIME;use_sys     =DEF_SYSLOG;
  log_con     =LOG_CONNECT; order       =DEF_ORDER;

  if(qotd_dir!=NULL){free(qotd_dir);} 
  if(tcp_port!=NULL){free(tcp_port);}
  if(udp_port!=NULL){free(udp_port);}
  if(def_user!=NULL){free(def_user);}
  if(log_file!=NULL){free(log_file);}
  if   (allow!=NULL){free(allow);}
  if    (deny!=NULL){free(deny);}


  log_file=(char *)xmalloc((strlen(DEF_LOG_FILE)+1)*sizeof(char));
  qotd_dir=(char *)xmalloc((strlen(DEF_FORTUNE)+1)*sizeof(char));
  tcp_port=(char *)xmalloc((strlen(DEF_TCP_PORT)+1)*sizeof(char));
  udp_port=(char *)xmalloc((strlen(DEF_UDP_PORT)+1)*sizeof(char));
  def_user=(char *)xmalloc((strlen(DEF_USER)+1)*sizeof(char));
  allow   =(char *)xmalloc((strlen(DEF_ALLOW)+1)*sizeof(char));
  deny    =(char *)xmalloc((strlen(DEF_DENY)+1)*sizeof(char));

  strcpy (qotd_dir, DEF_FORTUNE); 
  strcpy (tcp_port, DEF_TCP_PORT); 
  strcpy (udp_port, DEF_UDP_PORT); 
  strcpy (def_user, DEF_USER);
  strcpy (log_file, DEF_LOG_FILE);
  strcpy (allow,    DEF_ALLOW);
  strcpy (deny,     DEF_DENY);


}

char *config_getline(char *line,FILE *fptr)
{
  /* Gets next non-comment, non-blank line of config file */
  int  init_buffer   = 30;
  int  grow_buffer   = 30;
  int  buffer_pos    = 0;
  int  comment_flag  = 0;
  int  max_buffer    = 1024;
  char next_char;
  if (line!=NULL)
    {
      free(line);
    }
  line=(char *)xmalloc(init_buffer*sizeof(char));
  next_char=fgetc(fptr);
  while ((next_char!=EOF)&&(next_char!='\n'))
    {
      if (!comment_flag)
	{
	  line[buffer_pos]=next_char;
	  if (next_char!=' ')
	    {
	      buffer_pos++;
	    }
	  if (next_char=='#')
	    {
	      buffer_pos--;
	      comment_flag=1;
	    }
	  if (buffer_pos>=init_buffer)
	    {
	      line=xrealloc(line,init_buffer+grow_buffer);
	      init_buffer += grow_buffer;
	    }
	  if (buffer_pos>=max_buffer)
	    {
	      while ((next_char!=EOF)&&(next_char!='\n'))
		{
		  next_char=fgetc(fptr);
		}
	      return(line);
	    }
	}
      next_char=fgetc(fptr);
    }
  line[buffer_pos]=0;
  if ((line[0]=='#')||((buffer_pos==0)&&(!feof(fptr))))
    {
      line=config_getline(line,fptr);
    }
  return(line);
}

void config_readfile()
{
  char *line_buffer;
  char *part1,*part2;
  int   length,temp;
  FILE *cfile;

  line_buffer = NULL;
  config_setdefault();
  if (!(config_havefile))
    {
      return;
    }

  if(debug_flag)
    {
      my_message(MSG_INITSTART,"Starting config file read.","");
    }

  if((cfile=fopen(config_filename,"r"))==NULL)
    {
      /* Need to exit if files gone away */
      my_message(MSG_ERRFILEF,"Lost config file. Exiting.","");
    }

  while(!feof(cfile))
    {
      line_buffer=config_getline(line_buffer,cfile);
      length = strlen(line_buffer);
     
      if (length > 1)
	{
	  part1=(char *)xmalloc(length*sizeof(char));
	  part2=(char *)xmalloc(length*sizeof(char));
	  sscanf(line_buffer,"%[^=]=%s",part1,part2); 
	  if (!strcmp(part1,"port_tcp"))
	    {
	      temp=strlen(part2);
	      if ((temp>0)&&(temp<1024))
		{
		  if (tcp_port!=NULL){free(tcp_port);}
		  tcp_port=(char *)xmalloc((temp+1)*sizeof(char));
		  strcpy(tcp_port,part2);
		}
	    }
	  if (!strcmp(part1,"port_udp"))
	    {
	      temp=strlen(part2);
	      if ((temp>0)&&(temp<1024))
		{
		  if (udp_port!=NULL){free(udp_port);}
		  udp_port=(char *)xmalloc((temp+1)*sizeof(char));
		  strcpy(udp_port,part2);
		}
	    }
	  if (!strcmp(part1,"allow"))
	    {
	      temp=strlen(part2);
	      if ((temp>0)&&(temp<1024))
		{
		  if (allow!=NULL){free(allow);}
		  allow=(char *)xmalloc((temp+1)*sizeof(char));
		  strcpy(allow,part2);
		}
	    }
	  if (!strcmp(part1,"order"))
	    {
	      temp=atoi(part2);
	      order=(temp!=0);
	    }
	  if (!strcmp(part1,"deny"))
	    {
	      temp=strlen(part2);
	      if ((temp>0)&&(temp<1024))
		{ 
		  if (deny!=NULL){free(deny);} 
		  deny=(char *)xmalloc((temp+1)*sizeof(char)); 
		  strcpy(deny,part2); 
		}
	    }
	  if (!strcmp(part1,"log_msg"))
	    {
	      temp=atoi(part2);
	      if ((temp>=0)&&(temp<65535)){log_msg=temp;}
	    }
	  if (!strcmp(part1,"mark_time"))
	    {
	      temp=atoi(part2);
	      if ((temp>=0)&&(temp<65535)){mark_time=temp;}
	    }
	  if (!strcmp(part1,"max_tcp"))
	    {
	      temp=atoi(part2);
	      if ((temp>=0)&&(temp<65535)){max_con_tcp=temp;}
	    }
	  if (!strcmp(part1,"max_udp"))
	    {
	      temp=atoi(part2);
	      if ((temp>=0)&&(temp<65535)){max_con_udp=temp;}
	    }
	  if (!strcmp(part1,"syslog"))
	    {
	      temp=atoi(part2);
	      use_sys=(temp!=0);
#if !(HAVE_SYSLOG_H)
              use_sys=0;
#endif 	      
	    }
	  if (!strcmp(part1,"log_count"))
	    {
	      temp=atoi(part2);
	      log_count=(temp!=0);
	    }
	  if (!strcmp(part1,"log_con"))
	    {
	      temp=atoi(part2);
	      if ((temp>=0)&&(temp<65535)){log_con=temp;}
	    }
	  if (!strcmp(part1,"chk_con"))
	    {
	      temp=atoi(part2);
	      if ((temp>=0)&&(temp<65535)){chk_con=temp;}
	    }
	  if (!strcmp(part1,"abs_max"))
	    {
	      temp=atoi(part2);
	      if ((temp>0)&&(temp<65535)){abs_max=temp;}
	    } 
	  if (!strcmp(part1,"abs_min"))
	    {
	      temp=atoi(part2);
	      if ((temp>0)&&(temp<65535)){abs_min=temp;}
	    }
	  if (!strcmp(part1,"use_udp"))
	    {
	      temp=atoi(part2);
	      use_udp=(temp!=0);
	    }
	  if (!strcmp(part1,"use_tcp"))
	    {
	      temp=atoi(part2);
	      use_tcp=(temp!=0);
	    }
	  if (!strcmp(part1,"offensive"))
	    {
	      temp=atoi(part2);
	      use_off=(temp!=0);
	    }
	  if (!strcmp(part1,"rfc865"))
	    {
	      temp=atoi(part2);
	      use_rfc=(temp!=0);
	    }
	  if (!strcmp(part1,"daemon"))
	    {
	      temp=atoi(part2);
	      daemon_flag=(temp!=0);
	    }

	  if (!strcmp(part1,"directory"))
	    {
	      temp=strlen(part2);
	      if ((temp>0)&&(temp<1024))
		{
		  if (qotd_dir!=NULL){free(qotd_dir);}
		  qotd_dir=(char *)xmalloc((temp+1)*sizeof(char));
		  strcpy(qotd_dir,part2);
		}
	    }
	  if (!strcmp(part1,"user"))
	    {
	      temp=strlen(part2);
	      if ((temp>0)&&(temp<1024))
		{
		  if (def_user!=NULL){free(def_user);}
		  def_user=(char *)xmalloc((temp+1)*sizeof(char));
		  strcpy(def_user,part2);
		}
	    }
	  if (!strcmp(part1,"log_file"))
	    {
	      temp=strlen(part2);
	      if ((temp>0)&&(temp<1024))
		{
		  if (log_file!=NULL){free(log_file);}
		  log_file=(char *)xmalloc((temp+1)*sizeof(char));
		  strcpy(log_file,part2);
		}
	    }
	  free(part1);free(part2);
	}
    }
  if (line_buffer!=NULL)
    {
      free(line_buffer);
    }
  fclose(cfile);
  my_message(MSG_INITFINISH,"Finished reading config file.","");
}

